#ifndef _RDBCURSOR_H_
#define _RDBCURSOR_H_

// A RdbCursor is an iterator over a query.
// It stores some state, but mostly forwards its requests to the query.

#include "RdbError.h"
#include "RdbItem.h"
#include "String.h"
#include "Boolean.h"

class RdbQuery;			// forward decl.


class RdbCursor
: public RdbError
{
  public:

    // Note: this data is public for Rdb*Query's only!!
    // I should find a better way...
    // Two possible ways:
    //     list each derived Rdb*Query as a friend.
    //     make them protected, then each system must derived a cursor.
    // nope, i think i found a good way...

    RdbCursor( RdbQuery& newQuery, RdbItemList& newBindings );
				// create a cursor on this query and with
				// with these bindings.  The bindings must
				// match the query's adornment/types

    ~RdbCursor();

    ErrorCode GetNext( RdbItemList& result ); // get the next result

    void Invalidate();		// mark this cursor as invalid

    const RdbItemList& Bindings() const; // access bindings

  private:

    RdbQuery& query;		// the query to put cursor on

    RdbItemList bindings;	// bindings for this query.

    void* id;			// identifier to backend
				// This should be set by Query::VOpen
				// to whatever the backend needs to 
				// identify its session.
				// The Query::VClose will be called to
				// clean it up.

    Boolean ready;		// is the cursor opened, etc?

    RdbCursor(const RdbCursor&); // na

    RdbCursor& operator =(const RdbCursor&); // na

    friend class RdbQuery;	// to get to id
};


#include "RdbCursor.i"

#endif // _RDBCURSOR_H_
