#ifndef _RDBSYBASEPROCESS_H_
#define _RDBSYBASEPROCESS_H_


#include "RdbError.h"
#include "String.h"
#include "List.h"
#include "sybase.h"


// Forward Declarations:
class RdbSybaseDatabase;


// A process pool is a collection of open connections to a database.
// It allows connections to be reused, rather than openning & closing
// them all of the time.  More importantly, this will enable commits
// & rollbacks to work properly.
//
// Process pools are created in databases.

class RdbSybaseProcessPool
{
  public:
    
    RdbSybaseProcessPool(const String newDbName,
			 const String userid,
			 const String password,
			 const String newServerName);

    ~RdbSybaseProcessPool();

    RdbError::ErrorCode AllocateProcess(DBPROCESS*& dbproc,
					int& procid);

    RdbError::ErrorCode ReleaseProcess(int procid);
  
  private:

    typedef PtrList<DBPROCESS> ProcessList;

    static int total_inuse;

    static int limit;

    LOGINREC* login;

    String dbName;

    String serverName;

    ProcessList inuseProc;

    ProcessList availableProc;

};

//---------------------------------------------------------------------------

// A process controls the allocation/deallocation of the process within
// the pool.
//
// These can be created anywhere they are needed.

class RdbSybaseProcess
{
  public:

    RdbSybaseProcess(RdbSybaseDatabase& newDatabase);
				// opens a comm. process in the database

    ~RdbSybaseProcess();	// closes the process

    operator DBPROCESS*();	// access to dbproc
				// may be null, if there was an error
				// during open.

  private:
    
    RdbSybaseDatabase& database; // the database that the process pool is in

    int procid;			// the id of the process in the pool

    DBPROCESS* dbproc;		// the process that has been allocated

};


#include "RdbSybaseProcess.i"

#endif // _RDBSYBASEPROCESS_H_
