// *************************************************************************
//    BacktrackInfoArray.C
// *************************************************************************

#include "BacktrackInfoArray.h"

#include "RuleInfo.h"
#include "BindTime.h"
#include "Globals.h"
#include "Debug.h"

// *************************************************************************
Cor_BacktrackInfoArray::Cor_BacktrackInfoArray(Cor_RuleInfoArray *rArray)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    Cor_BindTime  bta;

    numRules = rArray->numRules;
    firstFailureBacktrackArrays = new int* [numRules];
    nextFailureBacktrackArrays = new int* [numRules];
    successBacktrackPoints = new int [numRules];

    for (int i = 0; i < numRules; i++) {

        bta.Analyze(rArray->rInfoPtrs[i]);

        if (rArray->rInfoPtrs[i]->num_literals) {
            firstFailureBacktrackArrays[i] =
                new int[rArray->rInfoPtrs[i]->num_literals];
            nextFailureBacktrackArrays[i] =
                new int[rArray->rInfoPtrs[i]->num_literals];

            bta.get_firstFailure_BT_Pt(firstFailureBacktrackArrays[i]);
            bta.get_nextFailure_BT_Pt(nextFailureBacktrackArrays[i]);
        } else {
            firstFailureBacktrackArrays[i] = NULL;
            nextFailureBacktrackArrays[i] = NULL;
        }

        successBacktrackPoints[i] = bta.get_Success_BT_Pt();
        bta.clear();
    }
}

// *************************************************************************
Cor_BacktrackInfoArray::~Cor_BacktrackInfoArray()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    for (int j=0; j < numRules; j++) {
        delete [] firstFailureBacktrackArrays[j];
        delete [] nextFailureBacktrackArrays[j];
    }

    delete [] firstFailureBacktrackArrays;
    delete [] nextFailureBacktrackArrays;
    delete [] successBacktrackPoints;
}

// *************************************************************************
// Alter backtrack points to implement naive backtracking
void Cor_BacktrackInfoArray::set_naive_backtracking(Cor_RuleInfoArray *rArray)
{
    for (int i = 0; i < numRules; i++) {

        if (rArray->rInfoPtrs[i]->num_literals) {

            int nl = rArray->rInfoPtrs[i]->num_literals;
            for (int j=0; j < nl; j++) {

                firstFailureBacktrackArrays[i][j] =
                        nextFailureBacktrackArrays[i][j];
            }
        }

        //successBacktrackPoints[i] = ??;
    }
}

// *************************************************************************
