// **************************************************************************
//    ParserStruct.h
// **************************************************************************

#ifndef _PARSERSTRUCT_H_
#define _PARSERSTRUCT_H_

#include "Rule.h"
#include "ModuleInfo.h"
#include "Table.h"
#include "RuleInfoArray.h"
#include "SemiNaive.h"
#include "PredAnnotations.h"
#include "ParseUtils.h"
#include "Stack.h"

// **************************************************************************

class Cor_ParserStruct;
class Cor_Predicate;
class Cor_Collection;
class Cor_CollectStack;
class Cor_SymTable;

typedef void (* ExecFunction) (Cor_ParserStruct*);

// **************************************************************************

class Cor_ParserStruct {

  public:
    ExecFunction          func;
    int                   label_counter;

    Cor_SemiNaive        *snrule;
    Cor_Table            *table;
    Cor_RuleInfoArray    *rInfo_arr;

    Cor_ModuleInfo       *CurModule;
    int                   nested_index;

    Cor_CArgStack         rule_var_names;
    Cor_CArgStack         sip_var_names;
    Cor_VoidStack         rhspreds;

            //  The rules are stored hierarchically in these things
    Cor_Clause           *cur_rule;
    Cor_Collection       *cur_collect;
    Cor_CollectStack     *all_rules;

    Cor_Predicate        *Predicates;
    Cor_SymTable         *PredicateTable;
    Cor_Predicate        *AllPredicates;

      // SPF - this is just to keep purify quiet about these
    Cor_Predicate        *Junk_Preds;

    Cor_PredAnnotations  *make_index_annotations;
    Cor_PredAnnotations  *agg_sel_annotations;
    Cor_PredAnnotations  *diskrel_annotations;
    Cor_PredAnnotations  *AllowedAdornList;
    Cor_PredAnnotations  *DisallowedAdornList;
    Cor_PredAnnotations  *asyncrel_annotations;

    // WARNING: In case of conflicts between AllowedAdornList and 
    //          DisallowedAdornList, the DisallowedAdornList is
    //          given priority, and the AllowedAdornList is over-ridden.

    Cor_PredAnnotations  *MultisetAnnotations;
    int                   lhs_flag;
    int                   agg_flag;

      // Only used for parameter passing with the C++ Interface
      // (this will be eliminated soon -SPF)
    int                   no_param_args;
    Cor_ArgList          *orig_args;

    void assign_func(ExecFunction exec_func) {
        func = exec_func;
    }

    void execute() { if (func) (*func)(this); }

      // reset() re-initializes the parserStruct
      // It deletes some memory, but if you're serious, it will delete more.
      //   (called by the parser at the end of each input)
    void reset(int serious = 1);

    Cor_ParserStruct();
   ~Cor_ParserStruct();

  private:
      // This clears the memory but does not reinitialize the parserStruct
    void clear(int serious = 1);

      // Do not call this directly, reset() should be used instead.
    void init();
};

// **************************************************************************

Cor_Predicate *Cor_FindPredicate(Cor_ParserStruct *Cor_parserStruct,
    Cor_CArg& name, int arity);
Cor_Predicate *Cor_LookupPredicate(Cor_ParserStruct *parserStruct,
    Cor_CArg& name, int arity);

// **************************************************************************

#endif _PARSERSTRUCT_H_
