// *************************************************************************
//    RelationMisc.h
// *************************************************************************

#ifndef _RelationMisc_H_
#define _RelationMisc_H_

#include "Unify.h"
#include "CArg.h"
#include "Debug.h"

// *************************************************************************

class Cor_StorageRelation;
class Cor_GenericIndex;
class Cor_AggSelInfo;

// *************************************************************************
// Abstract interface for handling "generic" Relations, independent
// of concrete implementation (linked list, database, builtin, etc).
// To add further defines, just number them with successive powers of 2 

// The following two can only be used within methods of GenericRel (or 
// methods of subclasses of GenericRel) 
#define REL_DISPLAY_NEW_INSERTIONS 1    // Display non-subsumed insertions
#define REL_DISPLAY_INSERTIONS     2    // Display all attempted insertions

// The following can only be used within methods of GenericIndex (or 
// methods of subclasses of GenericIndex) 
#define REL_DISPLAY_INDEXOPS       4    // Display calls to index operations

#define REL_PROFILE_STATS          8    // Generate profile statistics

#define REL_EXPLAIN               16    // Turn browser on (dump)

// *************************************************************************

#define DISPLAY_INSERTIONS  \
       ((Cor_exEnv->GlobalRelationOptions | this->local_options) & REL_DISPLAY_INSERTIONS)
#define DISPLAY_NEW_INSERTIONS  \
       ((Cor_exEnv->GlobalRelationOptions | this->local_options) & REL_DISPLAY_NEW_INSERTIONS)
                
#define DISPLAY_INDEXOPS  \
    ((Cor_exEnv->GlobalRelationOptions | this->relation->local_options) & REL_DISPLAY_INDEXOPS)
                
// *************************************************************************
//  iterator.ipos is used to keep track of where were are in the scan
//  of the hash index.  The possible values are:
#define Scanning_Init  0     // Initial state
#define Scanning_Known 1     // scanning known bucket
#define Scanning_Vars  2     // scanning from var_chain
#define Scanning_New   3     // scanning from delta chain
#define Scanning_Done  4     // done
#define Scanning_Hash  5

// *************************************************************************
// For i in [Scanning_Hash .. Scanning_Hash+hash_size-1] means
// that we are scanning all hash buckets, and are currently at
// bucket i-Scanning_Hash 

typedef int RelationOptions; // List of possible options follows.

// *************************************************************************

enum RelationKind {
    // the next three are the only possible rkind values possible for
    // actual materialized relations. Only these rkinds need to be
    // checked for at run time

    COR_R_MAGIC,
    COR_R_SUPPLEMENTARY,
    COR_R_ANSWER,
    COR_R_DERIVED,
    COR_R_EXODUS,
    COR_R_OTHER
};

// *************************************************************************

extern int Cor_RelCount;

// *************************************************************************

void Cor_group_dump(Cor_BindEnv *new_env, Cor_ArgList *new_arglist,
    Cor_Relation *rel, char *old_arglist = NULL);

  // Returns the basename, and gives pointers for magic and
  // adornments (or NULLs). The relname buffer passed in can be modified.
char *Cor_split_relname(char *relname, char **magic, char **adorn);

RelationKind  Cor_find_r_kind(Cor_CArg& name);

Cor_Relation *Cor_find_local_or_external_relation(Cor_CArg& name, int arity);
Cor_Relation *Cor_find_external_relation(Cor_CArg& name, int arity);
Cor_Relation *Cor_Find_External_Relation(Cor_CArg& name, int arity);

Cor_Relation *find_relation(char *name, int arity);
Cor_Relation *find_relation(Cor_CArg& name, int arity);
//Cor_Relation *find_relation(Cor_CArg c, int arity);

Cor_Relation *Cor_make_relation(Cor_CArg& name, int arity,
                 int delta_indexed = -1);
Cor_Relation *Cor_make_relation(char *name, int arity, int delta_indexed = -1);
Cor_StorageRelation *Cor_make_local_relation(Cor_CArg& name, int arity,
        int delta_indexed, int set_subsum = -1);

void Cor_reclaim_relation(Cor_StorageRelation *rel);
void Cor_reclaim_index(Cor_GenericIndex *index, RelationKind r_kind);

int Cor_duplicate_agg_sel_anno(Cor_StorageRelation *rel,
        Cor_AggSelInfo *newinfo);

void delete_id_tuple (Cor_Relation *rel, Cor_Tuple *tuple);
void delete_tuple (Cor_Relation *rel, Cor_Tuple *tuple);

extern void update_tuple (Cor_Tuple *tuple, int arg_pos, Cor_CArg& new_val);
extern void update_tuple (Cor_Relation *rel, Cor_Tuple *old_tuple,
        Cor_Tuple *new_tuple);

// *************************************************************************

#endif _RelationMisc_H_
