// *************************************************************************
//    SortedRelation.h
// *************************************************************************

#ifndef _SortedRelation_H_
#define _SortedRelation_H_

#include "StorageRelation.h"
#include "TupleNode.h"

class Cor_IntStack;

// *************************************************************************

class Cor_SortedRelation : public Cor_StorageRelation {

  private:

    int            _incr_order;  // Sort in increasing/decreasing order

    Cor_IntStack  *_sort_by;     // Argument numbers giving the order
                                 // to sort-by (in the order specified above)

    Cor_TupleNode *_list;        // Sorted list of tuples

    void       Init(Cor_TupleIterator& it);
    Cor_Tuple *Done(Cor_TupleIterator& it);

                            // Standard 3-way comparison function
                            // between two nodes.  Takes the order
                            // desired into account.
                            //     -1 : left comes before right (in a scan)
                            //      0 : they're equal (on the sort-by cols)
                            //      1 : right comes before left
    int Compare(Cor_TupleNode *left, Cor_TupleNode *right);
    
  public:

    Cor_SortedRelation(int arity, Cor_IntStack *sort_by, int incr_order=1);
   ~Cor_SortedRelation();

    virtual int insert_tuple(Cor_Tuple *tuple);
    
    virtual Cor_BindEnv *get_next(Cor_TupleIterator& it); 
    virtual Cor_Tuple *get_next_tuple( Cor_TupleIterator& it);

    virtual void release(Cor_TupleIterator& it);

    // Empty tuples out of the relation.  Delete the tuples
    // only if the deleteTuples arg is not 0
    virtual void empty_relation(int deleteTuples = 0);

    virtual void print_name(ostream *out);
};

// *************************************************************************

#endif // _SortedRelation_H_
